@@ -15,13 +15,13 @@ extern crate tempfile;
 use self::tempfile::Builder;
 
 #[derive(Debug)]
-/// Possible errors that can occur during `rustc --pretty=expanded`.
+/// Possible errors that can occur during `rustc -Zunpretty=expanded`.
 pub enum Error {
     /// Error during creation of temporary directory
     Io(io::Error),
     /// Output of `cargo metadata` was not valid utf8
     Utf8(Utf8Error),
-    /// Error during execution of `cargo rustc --pretty=expanded`
+    /// Error during execution of `cargo rustc -Zunpretty=expanded`
     Compile(String),
 }
 
@@ -92,6 +92,10 @@ pub fn expand(
 
     cmd.arg("rustc");
     cmd.arg("--lib");
+    // When build with the release profile we can't choose the `check` profile.
+    if profile != Profile::Release {
+        cmd.arg("--profile=check");
+    }
     cmd.arg("--manifest-path");
     cmd.arg(manifest_path);
     if let Some(features) = expand_features {
@@ -126,9 +130,7 @@ pub fn expand(
     cmd.arg(&package);
     cmd.arg("--verbose");
     cmd.arg("--");
-    cmd.arg("-Z");
-    cmd.arg("unstable-options");
-    cmd.arg("--pretty=expanded");
+    cmd.arg("-Zunpretty=expanded");
     info!("Command: {:?}", cmd);
     let output = cmd.output()?;
 
