@@ -4,8 +4,10 @@
 
 use std::io::Write;
 
+use crate::bindgen::config::Layout;
 use crate::bindgen::declarationtyperesolver::DeclarationType;
 use crate::bindgen::ir::{ConstExpr, Function, GenericArgument, Type};
+use crate::bindgen::language_backend::LanguageBackend;
 use crate::bindgen::writer::{ListType, SourceWriter};
 use crate::bindgen::{Config, Language};
 
@@ -20,15 +22,16 @@ enum CDeclarator {
         is_ref: bool,
     },
     Array(String),
-    Func(Vec<(Option<String>, CDecl)>, bool),
+    Func {
+        args: Vec<(Option<String>, CDecl)>,
+        layout: Layout,
+        never_return: bool,
+    },
 }
 
 impl CDeclarator {
     fn is_ptr(&self) -> bool {
-        match self {
-            CDeclarator::Ptr { .. } | CDeclarator::Func(..) => true,
-            _ => false,
-        }
+        matches!(self, CDeclarator::Ptr { .. } | CDeclarator::Func { .. })
     }
 }
 
@@ -38,6 +41,7 @@ struct CDecl {
     type_generic_args: Vec<GenericArgument>,
     declarators: Vec<CDeclarator>,
     type_ctype: Option<DeclarationType>,
+    deprecated: Option<String>,
 }
 
 impl CDecl {
@@ -48,6 +52,7 @@ impl CDecl {
             type_generic_args: Vec::new(),
             declarators: Vec::new(),
             type_ctype: None,
+            deprecated: None,
         }
     }
 
@@ -75,13 +80,13 @@ impl CDecl {
         cdecl
     }
 
-    fn from_func(f: &Function, layout_vertical: bool, config: &Config) -> CDecl {
+    fn from_func(f: &Function, layout: Layout, config: &Config) -> CDecl {
         let mut cdecl = CDecl::new();
-        cdecl.build_func(f, layout_vertical, config);
+        cdecl.build_func(f, layout, config);
         cdecl
     }
 
-    fn build_func(&mut self, f: &Function, layout_vertical: bool, config: &Config) {
+    fn build_func(&mut self, f: &Function, layout: Layout, config: &Config) {
         let args = f
             .args
             .iter()
@@ -92,8 +97,12 @@ impl CDecl {
                 )
             })
             .collect();
-        self.declarators
-            .push(CDeclarator::Func(args, layout_vertical));
+        self.declarators.push(CDeclarator::Func {
+            args,
+            layout,
+            never_return: f.never_return,
+        });
+        self.deprecated.clone_from(&f.annotations.deprecated);
         self.build_type(&f.ret, false, config);
     }
 
@@ -106,7 +115,7 @@ impl CDecl {
                         "error generating cdecl for {:?}",
                         t
                     );
-                    self.type_qualifers = "const".to_owned();
+                    "const".clone_into(&mut self.type_qualifers);
                 }
 
                 assert!(
@@ -114,13 +123,13 @@ impl CDecl {
                     "error generating cdecl for {:?}",
                     t
                 );
-                self.type_name = generic.export_name().to_owned();
+                generic.export_name().clone_into(&mut self.type_name);
                 assert!(
                     self.type_generic_args.is_empty(),
                     "error generating cdecl for {:?}",
                     t
                 );
-                self.type_generic_args = generic.generics().to_owned();
+                generic.generics().clone_into(&mut self.type_generic_args);
                 self.type_ctype = generic.ctype().cloned();
             }
             Type::Primitive(ref p) => {
@@ -130,7 +139,7 @@ impl CDecl {
                         "error generating cdecl for {:?}",
                         t
                     );
-                    self.type_qualifers = "const".to_owned();
+                    "const".clone_into(&mut self.type_qualifers);
                 }
 
                 assert!(
@@ -162,6 +171,7 @@ impl CDecl {
                 ref ret,
                 ref args,
                 is_nullable: _,
+                never_return,
             } => {
                 let args = args
                     .iter()
@@ -172,13 +182,23 @@ impl CDecl {
                     is_nullable: true,
                     is_ref: false,
                 });
-                self.declarators.push(CDeclarator::Func(args, false));
+                self.declarators.push(CDeclarator::Func {
+                    args,
+                    layout: config.function.args,
+                    never_return: *never_return,
+                });
                 self.build_type(ret, false, config);
             }
         }
     }
 
-    fn write<F: Write>(&self, out: &mut SourceWriter<F>, ident: Option<&str>, config: &Config) {
+    fn write<F: Write, LB: LanguageBackend>(
+        &self,
+        language_backend: &mut LB,
+        out: &mut SourceWriter<F>,
+        ident: Option<&str>,
+        config: &Config,
+    ) {
         // Write the type-specifier and type-qualifier first
         if !self.type_qualifers.is_empty() {
             write!(out, "{} ", self.type_qualifers);
@@ -194,7 +214,15 @@ impl CDecl {
 
         if !self.type_generic_args.is_empty() {
             out.write("<");
-            out.write_horizontal_source_list(&self.type_generic_args, ListType::Join(", "));
+            out.write_horizontal_source_list(
+                language_backend,
+                &self.type_generic_args,
+                ListType::Join(", "),
+                |language_backend, out, g| match *g {
+                    GenericArgument::Type(ref ty) => language_backend.write_type(out, ty),
+                    GenericArgument::Const(ref expr) => write!(out, "{}", expr.as_str()),
+                },
+            );
             out.write(">");
         }
 
@@ -231,7 +259,7 @@ impl CDecl {
                         out.write("(");
                     }
                 }
-                CDeclarator::Func(..) => {
+                CDeclarator::Func { .. } => {
                     if next_is_pointer {
                         out.write("(");
                     }
@@ -262,7 +290,11 @@ impl CDecl {
 
                     last_was_pointer = false;
                 }
-                CDeclarator::Func(ref args, layout_vertical) => {
+                CDeclarator::Func {
+                    ref args,
+                    ref layout,
+                    never_return,
+                } => {
                     if last_was_pointer {
                         out.write(")");
                     }
@@ -271,10 +303,16 @@ impl CDecl {
                     if args.is_empty() && config.language == Language::C {
                         out.write("void");
                     }
-                    if layout_vertical {
+
+                    fn write_vertical<F: Write, LB: LanguageBackend>(
+                        language_backend: &mut LB,
+                        out: &mut SourceWriter<F>,
+                        config: &Config,
+                        args: &[(Option<String>, CDecl)],
+                    ) {
                         let align_length = out.line_length_for_align();
                         out.push_set_spaces(align_length);
-                        for (i, &(ref arg_ident, ref arg_ty)) in args.iter().enumerate() {
+                        for (i, (arg_ident, arg_ty)) in args.iter().enumerate() {
                             if i != 0 {
                                 out.write(",");
                                 out.new_line();
@@ -283,11 +321,18 @@ impl CDecl {
                             // Convert &Option<String> to Option<&str>
                             let arg_ident = arg_ident.as_ref().map(|x| x.as_ref());
 
-                            arg_ty.write(out, arg_ident, config);
+                            arg_ty.write(language_backend, out, arg_ident, config);
                         }
                         out.pop_tab();
-                    } else {
-                        for (i, &(ref arg_ident, ref arg_ty)) in args.iter().enumerate() {
+                    }
+
+                    fn write_horizontal<F: Write, LB: LanguageBackend>(
+                        language_backend: &mut LB,
+                        out: &mut SourceWriter<F>,
+                        config: &Config,
+                        args: &[(Option<String>, CDecl)],
+                    ) {
+                        for (i, (arg_ident, arg_ty)) in args.iter().enumerate() {
                             if i != 0 {
                                 out.write(", ");
                             }
@@ -295,11 +340,30 @@ impl CDecl {
                             // Convert &Option<String> to Option<&str>
                             let arg_ident = arg_ident.as_ref().map(|x| x.as_ref());
 
-                            arg_ty.write(out, arg_ident, config);
+                            arg_ty.write(language_backend, out, arg_ident, config);
+                        }
+                    }
+
+                    match layout {
+                        Layout::Vertical => write_vertical(language_backend, out, config, args),
+                        Layout::Horizontal => write_horizontal(language_backend, out, config, args),
+                        Layout::Auto => {
+                            if !out.try_write(
+                                |out| write_horizontal(language_backend, out, config, args),
+                                config.line_length,
+                            ) {
+                                write_vertical(language_backend, out, config, args)
+                            }
                         }
                     }
                     out.write(")");
 
+                    if never_return && config.language != Language::Cython {
+                        if let Some(ref no_return_attr) = config.function.no_return {
+                            out.write_fmt(format_args!(" {}", no_return_attr));
+                        }
+                    }
+
                     last_was_pointer = true;
                 }
             }
@@ -307,19 +371,31 @@ impl CDecl {
     }
 }
 
-pub fn write_func<F: Write>(
+pub fn write_func<F: Write, LB: LanguageBackend>(
+    language_backend: &mut LB,
     out: &mut SourceWriter<F>,
     f: &Function,
-    layout_vertical: bool,
+    layout: Layout,
     config: &Config,
 ) {
-    CDecl::from_func(f, layout_vertical, config).write(out, Some(f.path().name()), config);
+    CDecl::from_func(f, layout, config).write(language_backend, out, Some(f.path().name()), config);
 }
 
-pub fn write_field<F: Write>(out: &mut SourceWriter<F>, t: &Type, ident: &str, config: &Config) {
-    CDecl::from_type(t, config).write(out, Some(ident), config);
+pub fn write_field<F: Write, LB: LanguageBackend>(
+    language_backend: &mut LB,
+    out: &mut SourceWriter<F>,
+    t: &Type,
+    ident: &str,
+    config: &Config,
+) {
+    CDecl::from_type(t, config).write(language_backend, out, Some(ident), config);
 }
 
-pub fn write_type<F: Write>(out: &mut SourceWriter<F>, t: &Type, config: &Config) {
-    CDecl::from_type(t, config).write(out, None, config);
+pub fn write_type<F: Write, LB: LanguageBackend>(
+    language_backend: &mut LB,
+    out: &mut SourceWriter<F>,
+    t: &Type,
+    config: &Config,
+) {
+    CDecl::from_type(t, config).write(language_backend, out, None, config);
 }
