@@ -5,10 +5,8 @@
 use std::fmt;
 use std::io::Write;
 
-use syn;
-
 use crate::bindgen::cargo::cargo_metadata::Dependency;
-use crate::bindgen::config::Config;
+use crate::bindgen::config::{Config, Language};
 use crate::bindgen::writer::SourceWriter;
 
 #[derive(PartialEq, Eq)]
@@ -83,6 +81,52 @@ impl fmt::Display for Cfg {
     }
 }
 
+impl syn::parse::Parse for Cfg {
+    fn parse(input: syn::parse::ParseStream) -> syn::Result<Self> {
+        let arg: syn::Meta = input.parse()?;
+
+        match arg {
+            syn::Meta::Path(path) => path
+                .get_ident()
+                .map(|ident| Cfg::Boolean(ident.to_string()))
+                .ok_or_else(|| input.error("path must be identifier")),
+            syn::Meta::NameValue(syn::MetaNameValue {
+                path,
+                value:
+                    syn::Expr::Lit(syn::ExprLit {
+                        lit: syn::Lit::Str(lit),
+                        ..
+                    }),
+                ..
+            }) => path
+                .get_ident()
+                .map(|ident| Cfg::Named(ident.to_string(), lit.value()))
+                .ok_or_else(|| input.error("path must be identifier")),
+            syn::Meta::List(meta) => {
+                if meta.path.is_ident("not") {
+                    let cfg = meta.parse_args()?;
+                    Ok(Cfg::Not(Box::new(cfg)))
+                } else if meta.path.is_ident("all") {
+                    let cfgs = meta.parse_args_with(
+                        syn::punctuated::Punctuated::<Cfg, syn::Token![,]>::parse_terminated,
+                    )?;
+
+                    Ok(Cfg::All(cfgs.into_iter().collect()))
+                } else if meta.path.is_ident("any") {
+                    let cfgs = meta.parse_args_with(
+                        syn::punctuated::Punctuated::<Cfg, syn::Token![,]>::parse_terminated,
+                    )?;
+
+                    Ok(Cfg::Any(cfgs.into_iter().collect()))
+                } else {
+                    Err(input.error("invalid list argument"))
+                }
+            }
+            _ => Err(input.error("Failed to parse cfg")),
+        }
+    }
+}
+
 impl Cfg {
     pub fn join(cfgs: &[Cfg]) -> Option<Cfg> {
         if cfgs.is_empty() {
@@ -105,12 +149,14 @@ impl Cfg {
         let mut configs = Vec::new();
 
         for attr in attrs {
-            if let Ok(syn::Meta::List(syn::MetaList { path, nested, .. })) = attr.parse_meta() {
-                if !path.is_ident("cfg") || nested.len() != 1 {
+            if let syn::Meta::List(meta @ syn::MetaList { path, .. }) = &attr.meta {
+                if !path.is_ident("cfg") {
                     continue;
                 }
 
-                if let Some(config) = Cfg::load_single(nested.first().unwrap()) {
+                let cfg = meta.parse_args().ok();
+
+                if let Some(config) = cfg {
                     configs.push(config);
                 }
             }
@@ -128,11 +174,11 @@ impl Cfg {
         match syn::parse_str::<syn::Meta>(target) {
             Ok(target) => {
                 // Parsing succeeded using the #[cfg] syntax
-                if let syn::Meta::List(syn::MetaList { path, nested, .. }) = target {
-                    if !path.is_ident("cfg") || nested.len() != 1 {
+                if let syn::Meta::List(meta) = target {
+                    if !meta.path.is_ident("cfg") {
                         return None;
                     }
-                    Cfg::load_single(nested.first().unwrap())
+                    meta.parse_args().ok()
                 } else {
                     None
                 }
@@ -140,86 +186,30 @@ impl Cfg {
             Err(_) => {
                 // Parsing failed using #[cfg], this may be a literal target
                 // name
-                Cfg::load_single(&syn::NestedMeta::Lit(syn::Lit::Str(syn::LitStr::new(
-                    target,
-                    proc_macro2::Span::call_site(),
-                ))))
+                Some(Cfg::Boolean(target.clone()))
             }
         }
     }
-
-    fn load_single(item: &syn::NestedMeta) -> Option<Cfg> {
-        Some(match *item {
-            syn::NestedMeta::Meta(syn::Meta::Path(ref path)) => {
-                Cfg::Boolean(format!("{}", path.segments.first().unwrap().ident))
-            }
-            syn::NestedMeta::Meta(syn::Meta::NameValue(syn::MetaNameValue {
-                ref path,
-                ref lit,
-                ..
-            })) => match *lit {
-                syn::Lit::Str(ref value) => Cfg::Named(
-                    format!("{}", path.segments.first().unwrap().ident),
-                    value.value(),
-                ),
-                _ => return None,
-            },
-            syn::NestedMeta::Meta(syn::Meta::List(syn::MetaList {
-                ref path,
-                ref nested,
-                ..
-            })) => {
-                if path.is_ident("any") {
-                    Cfg::Any(Cfg::load_list(nested.iter())?)
-                } else if path.is_ident("all") {
-                    Cfg::All(Cfg::load_list(nested.iter())?)
-                } else if path.is_ident("not") {
-                    if nested.len() != 1 {
-                        return None;
-                    }
-
-                    Cfg::Not(Box::new(Cfg::load_single(&nested[0])?))
-                } else {
-                    return None;
-                }
-            }
-            _ => return None,
-        })
-    }
-
-    fn load_list<'a, I: Iterator<Item = &'a syn::NestedMeta>>(attrs: I) -> Option<Vec<Cfg>> {
-        let mut configs = Vec::new();
-
-        for attr in attrs {
-            configs.push(Cfg::load_single(attr)?);
-        }
-
-        if configs.is_empty() {
-            None
-        } else {
-            Some(configs)
-        }
-    }
 }
 
 pub trait ToCondition: Sized {
     fn to_condition(&self, config: &Config) -> Option<Condition>;
 }
 
-impl<'a> ToCondition for Option<Cfg> {
+impl ToCondition for Option<Cfg> {
     fn to_condition(&self, config: &Config) -> Option<Condition> {
         self.as_ref()?.to_condition(config)
     }
 }
 
-impl<'a> ToCondition for Cfg {
+impl ToCondition for Cfg {
     fn to_condition(&self, config: &Config) -> Option<Condition> {
         match *self {
             Cfg::Boolean(ref cfg_name) => {
                 let define = config
                     .defines
                     .iter()
-                    .find(|(key, ..)| DefineKey::Boolean(&cfg_name) == DefineKey::load(key));
+                    .find(|(key, ..)| DefineKey::Boolean(cfg_name) == DefineKey::load(key));
                 if let Some((_, define)) = define {
                     Some(Condition::Define(define.to_owned()))
                 } else {
@@ -232,7 +222,7 @@ impl<'a> ToCondition for Cfg {
             }
             Cfg::Named(ref cfg_name, ref cfg_value) => {
                 let define = config.defines.iter().find(|(key, ..)| {
-                    DefineKey::Named(&cfg_name, &cfg_value) == DefineKey::load(key)
+                    DefineKey::Named(cfg_name, cfg_value) == DefineKey::load(key)
                 });
                 if let Some((_, define)) = define {
                     Some(Condition::Define(define.to_owned()))
@@ -285,15 +275,23 @@ impl Condition {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
         match *self {
             Condition::Define(ref define) => {
-                out.write("defined(");
-                write!(out, "{}", define);
-                out.write(")");
+                if config.language == Language::Cython {
+                    write!(out, "{}", define);
+                } else {
+                    out.write("defined(");
+                    write!(out, "{}", define);
+                    out.write(")");
+                }
             }
             Condition::Any(ref conditions) => {
                 out.write("(");
                 for (i, condition) in conditions.iter().enumerate() {
                     if i != 0 {
-                        out.write(" || ");
+                        out.write(if config.language == Language::Cython {
+                            " or "
+                        } else {
+                            " || "
+                        });
                     }
                     condition.write(config, out);
                 }
@@ -303,14 +301,22 @@ impl Condition {
                 out.write("(");
                 for (i, condition) in conditions.iter().enumerate() {
                     if i != 0 {
-                        out.write(" && ");
+                        out.write(if config.language == Language::Cython {
+                            " and "
+                        } else {
+                            " && "
+                        });
                     }
                     condition.write(config, out);
                 }
                 out.write(")");
             }
             Condition::Not(ref condition) => {
-                out.write("!");
+                out.write(if config.language == Language::Cython {
+                    "not "
+                } else {
+                    "!"
+                });
                 condition.write(config, out);
             }
         }
@@ -325,20 +331,30 @@ pub trait ConditionWrite {
 impl ConditionWrite for Option<Condition> {
     fn write_before<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
         if let Some(ref cfg) = *self {
-            out.push_set_spaces(0);
-            out.write("#if ");
-            cfg.write(config, out);
-            out.pop_set_spaces();
-            out.new_line();
+            if config.language == Language::Cython {
+                out.write("IF ");
+                cfg.write(config, out);
+                out.open_brace();
+            } else {
+                out.push_set_spaces(0);
+                out.write("#if ");
+                cfg.write(config, out);
+                out.pop_set_spaces();
+                out.new_line();
+            }
         }
     }
 
-    fn write_after<F: Write>(&self, _config: &Config, out: &mut SourceWriter<F>) {
+    fn write_after<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
         if self.is_some() {
-            out.new_line();
-            out.push_set_spaces(0);
-            out.write("#endif");
-            out.pop_set_spaces();
+            if config.language == Language::Cython {
+                out.close_brace(false);
+            } else {
+                out.new_line();
+                out.push_set_spaces(0);
+                out.write("#endif");
+                out.pop_set_spaces();
+            }
         }
     }
 }
